@@ -6,25 +6,31 @@
 # ----------
 
 from abc import abstractmethod
-from typing import Any
-from collections import ChainMap
+from typing import Any, List
 from contextlib import ExitStack
 
 from .err import ServiceNotFoundError
 from .symbols import Symbols
+from .ioc_services import ServicesMap
 from .ioc_resolver import IServiceInfoResolver, ServiceInfoChainResolver
 from .ioc_service_info import (
     LifeTime,
     IServiceInfo,
     ServiceInfo,
     ProviderServiceInfo,
+    GetAttrServiceInfo,
     ValueServiceInfo,
     GroupedServiceInfo,
     BindedServiceInfo,
+    CallerFrameServiceInfo
 )
 
 
 class IServiceProvider:
+    '''
+    the base interface for `ServiceProvider`.
+    '''
+
     @abstractmethod
     def __getitem__(self, key):
         raise NotImplementedError
@@ -46,11 +52,13 @@ class IServiceProvider:
 
 class ScopedServiceProvider(IServiceProvider):
 
-    def __init__(self, services: ChainMap):
+    def __init__(self, services: ServicesMap, parent=None):
         super().__init__()
         self._services = services
         self._exit_stack = None
-        self._services[Symbols.cache] = ValueServiceInfo({})
+        self._scoped_cache = {}
+        self._parent = parent
+        self.__class__ = ServiceProvider # hack
 
     def _get_service_info(self, key) -> IServiceInfo:
         try:
@@ -61,6 +69,9 @@ class ScopedServiceProvider(IServiceProvider):
         resolver: IServiceInfoResolver = self._services[Symbols.missing_resolver].get(self)
         return resolver.get(self, key)
 
+    def _get_service_info_list(self, key) -> List[IServiceInfo]:
+        return self._services.get_many(key)
+
     def __getitem__(self, key):
         service_info = self._get_service_info(key)
         try:
@@ -79,6 +90,27 @@ class ScopedServiceProvider(IServiceProvider):
                 return d
             raise
 
+    def get_many(self, key) -> List[Any]:
+        '''
+        get services by key.
+
+        ### example
+
+        when you registered multi services with the same key,
+        you can get them all:
+
+        ``` py
+        provider.register_value('a', 1)
+        provider.register_value('a', 2)
+        assert provider.get_many('a') == [2, 1] # rev order
+        ```
+        '''
+        service_infos = self._get_service_info_list(key)
+        try:
+            return [si.get(self) for si in service_infos]
+        except ServiceNotFoundError as err:
+            raise ServiceNotFoundError(key, *err.resolve_chain)
+
     def enter(self, context):
         '''
         enter the context.
@@ -98,7 +130,7 @@ class ScopedServiceProvider(IServiceProvider):
 
     def register_service_info(self, key, service_info: IServiceInfo):
         '''
-        register a IServiceInfo by key.
+        register a `IServiceInfo` by key.
         '''
         if not isinstance(service_info, IServiceInfo):
             raise TypeError('service_info must be instance of IServiceInfo.')
@@ -107,24 +139,36 @@ class ScopedServiceProvider(IServiceProvider):
     def register(self, key, factory, lifetime):
         '''
         register a service factory by key.
+
+        `factory` accept a function which require one or zero parameter.
+        if the count of parameter is 1, pass a `IServiceProvider` as the argument.
         '''
         return self.register_service_info(key, ServiceInfo(self, key, factory, lifetime))
 
     def register_singleton(self, key, factory):
         '''
-        register a singleton service factory by key.
+        register a service factory by key.
+
+        `factory` accept a function which require one or zero parameter.
+        if the count of parameter is 1, pass a `IServiceProvider` as the argument.
         '''
         return self.register(key, factory, LifeTime.singleton)
 
     def register_scoped(self, key, factory):
         '''
-        register a scoped service factory by key.
+        register a service factory by key.
+
+        `factory` accept a function which require one or zero parameter.
+        if the count of parameter is 1, pass a `IServiceProvider` as the argument.
         '''
         return self.register(key, factory, LifeTime.scoped)
 
     def register_transient(self, key, factory):
         '''
-        register a transient service factory by key.
+        register a service factory by key.
+
+        `factory` accept a function which require one or zero parameter.
+        if the count of parameter is 1, pass a `IServiceProvider` as the argument.
         '''
         return self.register(key, factory, LifeTime.transient)
 
@@ -168,23 +212,36 @@ class ScopedServiceProvider(IServiceProvider):
         '''
         create a scoped service provider.
         '''
-        return ScopedServiceProvider(self._services.new_child())
+        sp = ScopedServiceProvider(self._services.scope(), self)
+        self.enter(sp)
+        return sp
 
-    def decorator(self):
+    @property
+    def builder(self):
         '''
-        get a decorator helper for register items.
+        get a new `ServiceProviderBuilder` wrapper for this `ServiceProvider`.
         '''
-        from .decorate import ServiceProviderDecorator
-        return ServiceProviderDecorator(self)
+        from .builder import ServiceProviderBuilder
+        return ServiceProviderBuilder(self)
 
 
 class ServiceProvider(ScopedServiceProvider):
+    '''
+    the default impl for `IServiceProvider`.
+    '''
+
     def __init__(self):
-        super().__init__(ChainMap())
-        self._services[Symbols.provider] = ProviderServiceInfo()
+        super().__init__(ServicesMap())
+        provider_service_info = ProviderServiceInfo()
+        self._services[Symbols.provider] = provider_service_info
         self._services[Symbols.provider_root] = ValueServiceInfo(self)
+        self._services[Symbols.provider_parent] = GetAttrServiceInfo('_parent')
+        self._services[Symbols.cache] = GetAttrServiceInfo('_scoped_cache')
         self._services[Symbols.missing_resolver] = ValueServiceInfo(ServiceInfoChainResolver())
+        self._services[Symbols.caller_frame] = CallerFrameServiceInfo()
         # service alias
-        self._services['ioc'] = self._services[Symbols.provider]
-        self._services['provider'] = self._services[Symbols.provider]
-        self._services['service_provider'] = self._services[Symbols.provider]
+        self._services['ioc'] = provider_service_info
+        self._services['provider'] = provider_service_info
+        self._services['service_provider'] = provider_service_info
+        self._services[ServiceProvider] = provider_service_info
+        self._services[IServiceProvider] = provider_service_info
